<?php
/*
  Plugin Name: Tickera - Age Restriction per Ticket Type
  Plugin URI: http://tickera.com/
  Description: Custom add-on for DOB confirmation on the checkout
  Author: Tickera.com
  Author URI: http://tickera.com/
  Version: 1.0
  TextDomain: tc
  Domain Path: /languages/
  Copyright 2017 Tickera (http://tickera.com/)
 */

add_action('tc_load_additional_elements', 'tc_custom_form_date_picker_field_dob');

add_action('wp_enqueue_scripts', 'tc_custom_form_date_picker_field_front_header_dob');
add_action('admin_enqueue_scripts', 'tc_custom_form_date_picker_field_admin_scripts_dob');
add_filter('tc_ticket_fields', 'tc_custom_dob_tc_ticket_fields');
add_action('tc_cart_before_error_pass_check', 'tc_custom_dob_error_pass_check', 0, 2);

function get_age_from_date($birthDate) {
    $date_format_array = tc_get_custom_dob_date_format(true);

    $day_index = array_search('d', $date_format_array);
    $month_index = array_search('m', $date_format_array);
    $year_index = array_search('yy', $date_format_array);
    //explode the date to get month, day and year
    $birthDate = explode("/", $birthDate);
    //get age from date or birthdate
    $age = (date("md", date("U", mktime(0, 0, 0, $birthDate[$month_index], $birthDate[$day_index], $birthDate[$year_index]))) > date("md") ? ((date("Y") - $birthDate[$year_index]) - 1) : (date("Y") - $birthDate[$year_index]));
    return (int) $age;
}

function tc_custom_dob_error_pass_check($cart_error_number_orig, $tc_cart_errors_orig) {
    global $tc, $tc_cart_errors, $cart_error_number;

    $owner_data = array();
    $dob_error_messages = array();

    if (isset($_POST)) {
        foreach ($_POST as $field => $value) {
            if (preg_match('/owner_data_/', $field)) {
                $owner_data[str_replace(array('owner_data_', '_post_meta'), '', $field)] = $value;
            }
        }

        foreach ($owner_data as $field_name => $values) {
            $args = array(
                'post_type' => 'tc_form_fields',
                'numberposts' => 1,
                'meta_key' => 'name',
                'meta_value' => $field_name,
            );

            $form_fields = get_posts($args);

            $field_type = get_post_meta($form_fields[0]->ID, 'field_type', true);

            if ($field_type == 'tc_input_date_picker_field_form_element_dob') {//field is the dob check
                //check ticket type against the value provided
                $ticket_type_id = key($values);

                $age_restriction_type = get_post_meta($ticket_type_id, '_ticket_age_restriction_type', true);
                if (!isset($age_restriction_type) || empty($age_restriction_type)) {
                    $age_restriction_type = 'no';
                }

                if ($age_restriction_type !== 'no') {
                    switch ($age_restriction_type) {
                        case 'age_above'://get_age_from_date
                            $_age_above_value = (int) get_post_meta($ticket_type_id, '_age_above_value', true);
                            foreach ($values[$ticket_type_id] as $val) {
                                $age = get_age_from_date($val);
                                if ($age >= $_age_above_value) {
                                    //all OK
                                } else {
                                    $dob_error_messages[] = sprintf(__('%s ticket requires that attendee must be %s or older.', 'tc'), get_the_title($ticket_type_id), $_age_above_value);
                                    $cart_error_number++;
                                }
                            }
                            break;
                        case 'age_below':
                            $_age_below_value = (int) get_post_meta($ticket_type_id, '_age_below_value', true);
                            foreach ($values[$ticket_type_id] as $val) {
                                $age = get_age_from_date($val);
                                if ($age <= $_age_below_value) {
                                    //all OK
                                } else {
                                    $dob_error_messages[] = sprintf(__('%s ticket requires that attendee must be %s or younger.', 'tc'), get_the_title($ticket_type_id), $_age_below_value);
                                    $cart_error_number++;
                                }
                            }
                            break;
                        case 'age_between':
                            $_age_between_from_value = (int) get_post_meta($ticket_type_id, '_age_between_from_value', true);
                            $_age_between_to_value = (int) get_post_meta($ticket_type_id, '_age_between_to_value', true);
                            foreach ($values[$ticket_type_id] as $val) {
                                $age = get_age_from_date($val);
                                if ($age >= $_age_between_from_value && $age <= $_age_between_to_value) {
                                    //all OK
                                } else {
                                    $dob_error_messages[] = sprintf(__('%s ticket requires that attendee must be between %s and %s year old.', 'tc'), get_the_title($ticket_type_id), $_age_between_from_value, $_age_between_to_value);
                                    $cart_error_number++;
                                }
                            }
                            //
                            break;
                    }
                }
            }
        }

        $dob_error_messages_unique = array_unique($dob_error_messages);

        if (!empty($dob_error_messages_unique)) {
            foreach ($dob_error_messages_unique as $dob_error_messages_unique_message) {
                $tc_cart_errors .= '<li>' . $dob_error_messages_unique_message . '</li>';
            }
        }
    }
}

function tc_custom_dob_tc_ticket_fields($default_fields) {
    $default_fields[] = array(
        'field_name' => '_ticket_age_restriction_type',
        'field_title' => __('Age Restriction', 'tc'),
        'field_type' => 'function',
        'function' => 'tc_get_age_restrictions',
        'tooltip' => __('Choose if you want to restrict ticket type to a certain age. "Date of Birth" element must be added to a ticket type custom form in order to perform DOB check.', 'tc'),
        'table_visibility' => false,
        'post_field_type' => 'post_meta',
    );
    return $default_fields;
}

function tc_get_age_restrictions($field_name = '', $post_id = '') {
    if ($post_id !== '') {
        $currently_selected = get_post_meta($post_id, $field_name, true);
        if (empty($currently_selected)) {
            $currently_selected = 'no';
        }
    } else {
        $currently_selected = 'no';
    }

    $age_above_selected = get_post_meta($post_id, '_age_above_value', true);
    $age_above_selected = !isset($age_above_selected) || empty($age_above_selected) ? 18 : $age_above_selected;

    $age_below_selected = get_post_meta($post_id, '_age_below_value', true);
    $age_below_selected = !isset($age_below_selected) || empty($age_below_selected) ? 18 : $age_below_selected;

    $age_between_from_selected = get_post_meta($post_id, '_age_between_from_value', true);
    $age_between_from_selected = !isset($age_between_from_selected) || empty($age_between_from_selected) ? 18 : $age_between_from_selected;

    $age_between_to_selected = get_post_meta($post_id, '_age_between_to_value', true);
    $age_between_to_selected = !isset($age_between_to_selected) || empty($age_between_to_selected) ? 21 : $age_between_to_selected;
    ?>
    <label><input type="radio" name="_ticket_age_restriction_type_post_meta" value="no" <?php checked($currently_selected, 'no', true); ?> /><?php _e('No restriction', 'tc'); ?></label><br /><br />

    <label><input type="radio" name="_ticket_age_restriction_type_post_meta" value="age_above" <?php checked($currently_selected, 'age_above', true); ?> />

        <select name="_age_above_value_post_meta" id="_age_above_value_post_meta">
            <?php
            for ($age = 1; $age <= 99; $age++) {
                ?>
                <option value="<?php echo esc_attr($age); ?>" <?php selected($age, $age_above_selected, true); ?>><?php echo $age; ?></option>
                <?php
            }
            ?>
        </select> <?php _e(' and above', 'tc'); ?>
    </label><br /><br />

    <label><input type="radio" name="_ticket_age_restriction_type_post_meta" value="age_below" <?php checked($currently_selected, 'age_below', true); ?> />

        <select name="_age_below_value_post_meta" id="_age_below_value_post_meta">
            <?php
            for ($age = 1; $age <= 99; $age++) {
                ?>
                <option value="<?php echo esc_attr($age); ?>" <?php selected($age, $age_below_selected, true); ?>><?php echo $age; ?></option>
                <?php
            }
            ?>
        </select> <?php _e(' and below', 'tc'); ?>
    </label><br /><br />

    <label>
        <input type="radio" name="_ticket_age_restriction_type_post_meta" value="age_between" <?php checked($currently_selected, 'age_between', true); ?> />

        <select name="_age_between_from_value_post_meta" id="_age_between_from_value_post_meta">
            <?php
            for ($age = 1; $age <= 99; $age++) {
                ?>
                <option value="<?php echo esc_attr($age); ?>" <?php selected($age, $age_between_from_selected, true); ?>><?php echo $age; ?></option>
                <?php
            }
            ?>
        </select> 

        <?php _e(' to ', 'tc'); ?>

        <select name="_age_between_to_value_post_meta" id="_age_between_to_value_post_meta">
            <?php
            for ($age = 1; $age <= 99; $age++) {
                ?>
                <option value="<?php echo esc_attr($age); ?>" <?php selected($age, $age_between_to_selected, true); ?>><?php echo $age; ?></option>
                <?php
            }
            ?>
        </select>
    </label><br /><br />
    <?php
}

function tc_get_custom_dob_date_format($return_array = false) {
    //change order of format_array to get different format
    $format_array[] = 'd';
    $format_array[] = 'm';
    $format_array[] = 'yy';
    if ($return_array) {
        return $format_array;
    } else {//return string
        return implode('/', $format_array);
    }
}

function tc_custom_form_date_picker_field_admin_scripts_dob() {
    wp_enqueue_style('addon-style', plugins_url('includes/css/style.css', __FILE__));
}

function strip_array_indices_dob($ArrayToStrip) {
    foreach ($ArrayToStrip as $objArrayItem) {
        $NewArray[] = $objArrayItem;
    }

    return( $NewArray );
}

function tc_custom_form_date_picker_field_front_header_dob() {
    global $wp_locale;

    wp_enqueue_script('jquery-ui-datepicker');
    wp_enqueue_style('jquery-style', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/themes/smoothness/jquery-ui.css');

    wp_register_script('tc-custom-form-date-picker-main', plugins_url('includes/js/main.js', __FILE__), '', '', true);
    wp_enqueue_script('tc-custom-form-date-picker-main');

    $aryArgs = array(
        'closeText' => __('Done', 'tc'),
        'currentText' => __('Today', 'tc'),
        'monthNames' => strip_array_indices_dob($wp_locale->month),
        'monthNamesShort' => strip_array_indices_dob($wp_locale->month_abbrev),
        'monthStatus' => __('Show a different month', 'tc'),
        'dayNames' => strip_array_indices_dob($wp_locale->weekday),
        'dayNamesShort' => strip_array_indices_dob($wp_locale->weekday_abbrev),
        'dayNamesMin' => strip_array_indices_dob($wp_locale->weekday_initial),
        'dateFormat' => dateStringToDatepickerFormat_dob(tc_get_custom_dob_date_format()),
        'firstDay' => get_option('start_of_week'),
    );

    wp_localize_script('tc-custom-form-date-picker-main', 'objectL10n', $aryArgs);
}

function dateStringToDatepickerFormat_dob($dateString) {
    $pattern = array(
        //day
        'd', //day of the month
        'j', //3 letter name of the day
        'l', //full name of the day
        'z', //day of the year
        //month
        'F', //Month name full
        'M', //Month name short
        'n', //numeric month no leading zeros
        'm', //numeric month leading zeros
        //year
        'Y', //full numeric year
        'y'  //numeric year: 2 digit
    );
    $replace = array(
        'dd', 'd', 'DD', 'o',
        'MM', 'M', 'm', 'mm',
        'yy', 'y'
    );
    foreach ($pattern as &$p) {
        $p = '/' . $p . '/';
    }
    return preg_replace($pattern, $replace, $dateString);
}

function tc_custom_form_date_picker_field_dob() {

    class tc_input_date_picker_field_form_element_dob extends TC_Form_Elements {

        var $element_name = 'tc_input_date_picker_field_form_element_dob';
        var $element_title = 'Date Picker';
        var $element_type = 'text';
        var $element_html_class_name = 'tc_custom_date_picker_element_dob';

        function on_creation() {
            global $wp_query;

            $this->element_title = apply_filters('tc_input_date_field_form_element_title', __('Date of Birth', 'tc'));
            $this->field_name = isset($this->id) ? get_post_meta($this->id, 'name', true) : '';

            $post = get_post($this->id);
            $title = isset($post->post_title) ? $post->post_title : '';

            $this->field_label = isset($this->id) ? $title : '';
        }

        function date_format_php_to_js($sFormat) {
            switch ($sFormat) {
                //Predefined WP date formats
                case 'F j, Y':
                    return( 'MM dd, yy' );
                    break;
                case 'Y/m/d':
                    return( 'yy/mm/dd' );
                    break;
                case 'm/d/Y':
                    return( 'mm/dd/yy' );
                    break;
                case 'd/m/Y':
                    return( 'dd/mm/yy' );
                    break;
            }
        }

        function save($form_id) {
            $element_data = $_POST[$this->element_name];
            $elements_count = (count($element_data['field_name']) - 1);

            for ($i = 0; $i <= $elements_count; $i++) {
                if ((isset($element_data['field_label'][$i]) && !empty($element_data['field_label'][$i])) || (isset($element_data['field_name'][$i]) && !empty($element_data['field_name'][$i]))) {
                    $post_data = array(
                        'post_status' => 'publish',
                        'post_parent' => $form_id,
                        'post_title' => isset($element_data['field_label'][$i]) ? $element_data['field_label'][$i] : '',
                        'post_name' => isset($element_data['field_name'][$i]) ? $element_data['field_name'][$i] : sanitize_key($element_data['field_label'][$i]),
                        'post_content' => isset($element_data['field_values'][$i]) ? $element_data['field_values'][$i] : '',
                        'post_type' => $this->post_type
                    );

                    if (isset($element_data['field_post_id'][$i]) && is_numeric($element_data['field_post_id'][$i])) {
                        $post_data['ID'] = $element_data['field_post_id'][$i];
                    }

                    $post_id = wp_insert_post($post_data);

                    $post_metas = array(
                        'placeholder' => isset($element_data['field_placeholder'][$i]) ? $element_data['field_placeholder'][$i] : '',
                        'description' => isset($element_data['field_description'][$i]) ? $element_data['field_description'][$i] : '',
                        'default_value' => isset($element_data['field_default_values'][$i]) ? $element_data['field_default_values'][$i] : '',
                        'required' => isset($element_data['field_required'][$i]) ? $element_data['field_required'][$i] : '',
                        'export' => isset($element_data['field_export'][$i]) ? $element_data['field_export'][$i] : '',
                        'show_in_checkin_app' => isset($element_data['field_show_in_checkin_app'][$i]) ? $element_data['field_show_in_checkin_app'][$i] : '',
                        'as_ticket_template' => isset($element_data['field_as_ticket_template'][$i]) ? $element_data['field_as_ticket_template'][$i] : '',
                        'order_details' => isset($element_data['field_order_details'][$i]) ? $element_data['field_order_details'][$i] : '',
                        'order_column' => isset($element_data['field_order_column'][$i]) ? $element_data['field_order_column'][$i] : '',
                        'order' => isset($element_data['field_order'][$i]) ? $element_data['field_order'][$i] : '',
                        'row' => isset($element_data['field_row'][$i]) ? $element_data['field_row'][$i] : '',
                        'field_type' => $this->element_name,
                    );

                    //Make sure that we set field name only once so we don't loose data upon changing field label
                    $name = get_post_meta($post_id, 'name', true);

                    if (!isset($name) || (isset($name) && empty($name))) {
                        $post_metas['name'] = sanitize_key(isset($element_data['field_label'][$i]) ? str_replace('-', '_', $element_data['field_label'][$i]) . '_tcfn_' . rand(1, 9999) : 'tc_field_name_' . rand(1, 9999) );
                    }

                    foreach ($post_metas as $meta_key => $meta_value) {
                        update_post_meta($post_id, $meta_key, $meta_value);
                    }
                }
            }
        }

        function admin_order_details_page_value() {
            $value = array(
                'id' => $this->standard_field_name($this->element_name, true),
                'field_name' => 'tc_cart_info',
                'field_title' => $this->standard_field_label($this->element_name, true),
                'field_type' => 'function',
                'function' => 'tc_get_input_admin_order_details_page_value',
                'field_description' => '',
                'table_visibility' => false,
                'post_field_type' => 'post_meta'
            );

            return isset($value) && !empty($value) ? $value : '-';
        }

        function admin_content() {
            $this->standard_fields($this->element_name);
            $this->common_fields($this->element_name);
        }

    }

    tc_register_form_element('tc_input_date_picker_field_form_element_dob', __('Date of Birth', 'tc'));
}
?>